# Copyright 2011-2012 Paul Wouters <paul@nohats.ca>
# Based on http://code.google.com/p/geolocate-cli/ by Francis Markham
#
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License as published by 
# the Free Software Foundation, either version 3 of the License, or 
# (at your option) any later version. This program is distributed in the 
# hope that it will be useful, but WITHOUT ANY WARRANTY; without 
# even the implied warranty of MERCHANTABILITY or FITNESS FOR 
# A PARTICULAR PURPOSE. See the GNU General Public License 
# for more details. You should have received a copy of the GNU General 
# Public License along with this program. If not, see <http://www.gnu.org/licenses/>. 

"""  Obtain geo information via Google's API
This module allows you to obtain geo information about your current location.
It uses NetworkManager to obtain a list of wifi networks, and queries the
Google location service at:

https://maps.googleapis.com/maps/api/browserlocation/json?browser=firefox&sensor=false

For Terms of Service, see: https://developers.google.com/maps/faq#tos

There is a business use API which can take cell/wifi tower submissions, but it is
not free. You need to get an API key: 
https://developers.google.com/maps/documentation/business/geolocation/


Example use
-----------

>>> import geome
>>> geome.location()

or if you have a google api key

>>> import geome
>>> geome.apikey("YOURAPIKEY")
>>> geome.location()

"""

#import pprint
import dbus, sys, os, urllib2
try:
    import json
except ImportError:
    import simplejson as json 

__version__ = "2.0"
__api_key = ""
__api_url_key = "https://www.googleapis.com/geolocation/v1/geolocate?key="
__api_url_nokey = "https://maps.googleapis.com/maps/api/browserlocation/json?browser=firefox&sensor=false"
__api_revurl = "http://maps.googleapis.com/maps/api/geocode/json?sensor=false&latlng="
__useragent = "Mozilla/5.0"

def locate():
	location()

def apikey(apikey=""):
	global __api_key
	if apikey:
		__api_key = apikey
	else:
		keyfile = "%s/.config/geome/apikey"%os.path.expanduser("~")
		if os.path.isfile(keyfile):
			try:
				keyfile = open(keyfile,"r")
				__api_key = keyfile.read().strip()
			except:
				__api_key = ""
		return __api_key
		

def location():
	global __api_key
	__post_str = ""
	__aps = []
	if not __api_key:
		keyfile = "%s/.config/geome/apikey"%os.path.expanduser("~")
		if os.path.isfile(keyfile):
			try:
				keyfile = open(keyfile,"r")
				__api_key = keyfile.read().strip()
			except:
				__api_key = ""
	if __api_key:
		# without API key we cannot use the extended API that allows us to submit wifi/celltower info
		# print "Using API key %s"%__api_key
		__NM_DEVICE_TYPE_802_11_WIRELESS = 2
		__devices = ""
		try:
			__bus = dbus.SystemBus()
			__nm = __bus.get_object('org.freedesktop.NetworkManager', '/org/freedesktop/NetworkManager')
			__devices = __nm.GetDevices()
		except:
			pass

		__post_str = "{"
		for __path in __devices:
			__device = __bus.get_object('org.freedesktop.NetworkManager', __path)
			__device_props = dbus.Interface(__device, dbus_interface='org.freedesktop.DBus.Properties')
			__props = __device_props.GetAll("org.freedesktop.NetworkManager.Device")

			# Different magic seems to be required on different versions of networkmanager
			try:
				__devtype = __props["DeviceType"]
			except KeyError:
				__devtype = device.Get("org.freedesktop.NetworkManager.Device", 'DeviceType')

			if __devtype != __NM_DEVICE_TYPE_802_11_WIRELESS:
	                    	continue

			__access_points = __device.GetAccessPoints()

			for __network_path in __access_points:
				if not __network_path:
					# No active network
					continue
				__network = __bus.get_object('org.freedesktop.NetworkManager', __network_path)
				__network_props = dbus.Interface(__network, dbus_interface='org.freedesktop.DBus.Properties')
				__ssid = __network_props.Get("org.freedesktop.NetworkManager.AccessPoint", "Ssid", byte_arrays=True)
				__mac = __network_props.Get("org.freedesktop.NetworkManager.AccessPoint", "HwAddress")
				__sig_str = __network_props.Get("org.freedesktop.NetworkManager.AccessPoint", "Strength")
				__aps.append({"ssid": str(__ssid), "mac": str(__mac), "signal_strength": int(__sig_str)})


		if __aps:
			__post_str += '"wifiAccessPoints": ['
			for __ap in __aps:
				apstr = ""
				# google no longer wants ESSID, guess these change too often?
				if 'mac' in __ap and 'signal_strength' in __ap:
					apstr = '{"macAddress":"%s","signalStrength":%s'%(__ap['mac'],__ap['signal_strength'])
					# does NM support these?
					if 'age' in __ap:
						apstr += ',"age":%s',__ap['age']
					if 'channel' in __ap:
						apstr += ',"channel":%s',__ap['channel']
					if 'signalToNoiseRatio' in __ap:
						apstr += ',"signalToNoiseRatio":%s',__ap['signalToNoiseRatio']
					apstr += '},'
					__post_str += apstr

			__post_str = __post_str[:-1] # remove last comma
			__post_str += ']'
		else:
			__post_str += '"wifiAccessPoints": []'

		#if list_celltowers:
			# not yet does NM support these?
		

		__post_str += '}'
		# pprint.pprint(__post_str)
		__req = urllib2.Request(__api_url_key + __api_key, __post_str, {'User-agent' : __useragent })
		__req.add_header('Content-Type', 'application/json')
	else:
		# no api key
		# print "no API key, not submitting Wifi/Cell data"
		# pprint.pprint(__post_str)
		__req = urllib2.Request(__api_url_nokey, None, {'User-agent' : __useragent })

	try:
		__u = urllib2.urlopen(__req)
	except:
		return
	return  json.loads(__u.read())

def my_latlong():
	latlong = location()
	if latlong:
		# pprint.pprint(latlong)
		try:
			return latlong["location"]["lat"], latlong["location"]["lng"]
		except:
			pass
	return

def my_latlng():
	return my_latlong()

# Lookup a formatted address according to lat,long obtained from geome.location()
def json_latlong(lat="",long=""):
	if not lat or not long:
		lat, long = my_latlong()
	__url = "%s%s,%s"%(__api_revurl,lat,long)
	# print __url
	try:
		__req = urllib2.Request(__url, None, {'User-agent' : __useragent })
		__u = urllib2.urlopen(__req)
	except:
		sys.stderr.write("API query failure - wrong API key?\n")
		return
	return json.loads(__u.read())

def my_component(lat="",long="", qtype="formatted_address"):
	if not lat or not long:
		lat, long = my_latlong()
	try:
		json =  json_latlong(lat,long)
	except:
		return
	if not json:
		return
	for result in json["results"]:
		if result["geometry"]["location_type"] == "APPROXIMATE":
			if qtype == "formatted_address":
				return result["formatted_address"]
			for component in result["address_components"]:
				if qtype == "iso":
					iso = True
					qtype = "country"
				else:
					iso = False
				if qtype in component["types"]:
					if iso:
						return component["short_name"]
					else:
						return component["long_name"]


def my_address():
	return my_component(qtype = "formatted_address")

def my_country_iso():
	return my_component(qtype = "iso")

def my_country():
	return my_component(qtype = "country")

def my_city():
	return my_component(qtype = "locality")

def my_neighborhood():
	return my_component(qtype = "neighborhood")

def version():
	return __version__

